# a6_subclasses.py
# PUT YOUR NETID(S) HERE
# Sources/people consulted: FILL IN OR WRITE "NONE"
# PUT DATE YOU COMPLETED THIS HERE
# Skeleton by Prof. Lee, Apr 2022; typo in Ad Loop docstring fixed Apr 30 6pm.
# BEGIN REMOVE
# Thanks to Tiffany Sarver, Alanna Zhou, and a student for corrections.
# Thanks to Shawn Ong for alternate solution.
# END REMOVE

from a6_classes import Song, Mix, Loop


class Ad(Song):
    """An Ad should NOT be added to the Song class's all_of_em variable,
    because they aren't legitimate Songs. For instance, we wouldn't want them
    displayed in a catalog of Songs.

    Every Ad has artist "[Advertiser]" (literally, with the square brackets.)

    Class attribute:
        all_of_em: list of all Ads that are created.

    Instance attributes:
        includes those for Songs.

    """
    pass

    # STUDENTS: 
    # 1. Create methods, attributes and functions as appropriate. 
    # All new functions/methods should have good docstrings (including 
    # documenting each parameter), and new attributes should be mentioned in
    # the class docstring.
    #
    # 2. Explain your choices of function/method/attribute additions in one or 
    # more comment blocks starting with the all-caps word JUSTIFICATION.
    #
    # 3. For each method you add that over-rides one in the parent class, should
    # it include a call to super()? Explain this in your JUSTIFICATION comments.

    # BEGIN REMOVE
    # JUSTIFICATION: 
    #  1. We should keep track of all Ads, but they shouldn't be counted in
    #     all Songs. So we can't use the Song __init__ as is.  To save work,
    #     we call it in Ad's __init__ via super(), and then remove the new Ad 
    #     from Song.all_of_em and append it to our list of all ads.
    #  2. No need to override or shadow any other part of parent class Song.


    all_of_em = [] # Contains all Ads that are created.

    # Students did not need to create this class attribute.
    ARTIST = '[Advertiser]'

    def __init__(self, t, url=None):
        """"A new Ad with title `t`, artist "[Advertiser]", url `url`.

        This Ad will NOT be added to Songs.all_of_em, but WILL be added to
        Ad.all_of_em.

        Preconditions: 
            t: non-empty string 
            url: URL string or None

        """
        super().__init__(t, Ad.ARTIST, yt=url)
        # Now take self out of the Songs list.
        Song.all_of_em.remove(self) # We assume no other Ads are in that list.
        Ad.all_of_em.append(self)


        # An alternate solution would be to not call super() at all, but
        # copy over most of the contents of Song.__init__.

    # END REMOVE
    

class AdLoop(Loop):
    """
    Like a Loop, except that it plays an Ad after every Song.  The Ads should 
    appear in the order they occur in Ad.all_of_em.

    Example: suppose the source Mix has only two Songs, s1 and s2, and
    Ad.all_of_em has Ads a1, a2, a3, a4, a5.
    Then, the sequence of Songs occurring if one ran the play() method multiple
    times is:
       s1 a1 s2 a2 s1 a3 s2 a4 s1 a5 s2 a1 s1 a2 ... # typo fixed 4/30 (s2->s1)

    Instance attributes:
        Includes those of Loops.
        # BEGIN REMOVE
        Also,
        next_ad_pos: position of next Ad in Ad.all_of_em to be played.

        # END REMOVE

    """
    pass

    # STUDENTS: see instructions for class Ad.
    # Additionally: STUDENTS, document in a JUSTIFICATION comment block
    # your choice for what happens to the order of the Ads if the user decides 
    # to shuffle.

    # BEGIN REMOVE
    # JUSTIFICATION: 
    # Each instance has new attribute next_ad_pos, which acts analogously to
    # to next_pos but for Ad.all_of_em. 
    #
    # The __init__ calls super() so that title, slist, and next_pos can be
    # set up by that code.  Our __init__ just needs to do some additional work.
    #
    # We have to over-ride the play method to include the Ads. We can't use
    # super() because the interleaving of the Ads can't really be done before
    # or after calling the play method of Loop.
    #
    # Also made a new _adloop_prompt function that takes a next object and asks 
    # user if they want to hear it.  The next object should be a Song or an Ad.
    # Since the next object isn't closely associated with the AdLoop, made 
    # _adloop_prompt a NON-method function (we wouldn't have any use for the 
    # "self".)
    #
    # I decided not to shuffle the Ads even if the Songs in slist are shuffled.
    # Reasoning: advertisers might pay extra to have their ads shown earlier in
    # people's AdLoops.


    # Sketch of alternate solution for play() by Shawn Ong: "pad out both lists 
    # to be the same length (e.g. slist * len(adlist) and adlist * len(slist)) 
    # and then merge the two. From here it should be fine to treat it like a 
    # normal Loop"

    def __init__(self, m, shuffle=False):
        """
        A new AdLoop with its `slist` set to m.songs(), its `title` set to the
        title of `m`, its `next_pos` and `next_ad_pos` set to 0.

        EXCEPTION: If `shuffle` is True, then random.shuffle() should be applied 
        to the contents of `slist` to put it in random order.

        Preconditions: 
        `m` is a Mix (not None).
        `shuffle` is a boolean.
        """        
        super().__init__(m, shuffle=shuffle)
        self.next_ad_pos = 0

    def play(self, use_browser=True):
        """
        1. Print the title of this Loop.

        2. Continuously query the user if they want to play the next Song/Ad,
        where Ads are interleaved with Songs.
        If the last Song/Ad has been reached, the next song is the *first* Song
        in this Loop's slist or *first* Song in Ad.all_of_em.  

        Response of `q` means quit, any other response means play the next
        Song/Ad.

        3. If use_browser is False, then playing a Song yields a printout
        rather than causes a browser window to open.

        Precondition (added later): Ad.all_of_em is not empty.  
        """

        print('Starting Loop '+self.title)

        next_s = self.slist[self.next_pos]
        response = input(_adloop_prompt(next_s))
        while 'q' not in response:
            # First process a Song, then process the next Ad.
            # So every time we re-enter this loop, the next thing to play 
            # should be a Song. (Notice that there are two promptings
            # that happen in this while-loop, which means one doesn't need
            # to keep track of whether the next thing to play is a Song or an
            # Ad.

            # Song processing
            next_s.play(use_browser=use_browser)
            if self.next_pos == len(self.slist)-1:
                self.next_pos = 0  # Wrap around
            else:
                self.next_pos += 1

            # Ad processing
            next_ad = Ad.all_of_em[self.next_ad_pos]
            response = input(_adloop_prompt(next_ad))
            if 'q' not in response:
                next_ad.play(use_browser=use_browser)
                if self.next_ad_pos == len(Ad.all_of_em)-1:
                    self.next_ad_pos = 0 # Wrap around
                else:
                    self.next_ad_pos += 1

                # Set up for next Song
                next_s = self.slist[self.next_pos]
                response = input(_adloop_prompt(next_s))
        print('Exiting this Loop')


def _adloop_prompt(next_object):
    """Returns string prompting whether the user wants to continue to hear
    `next_object`, which should be a Song or an Ad."""
    return 'Hit Enter/Return to start the next Song, ' + \
            next_object.title + ' by ' + next_object.artist + ', ' + \
            'or "q" to quit: '

    # END REMOVE
