# a6_subclasses.py
# Alternate solution by TA Shawn Ong
#  (Justifications implicit in comments, since Shawn did this as volunteer effort)
# Skeleton by Prof. Lee, Apr 2022; typo in Ad Loop docstring fixed Apr 30 6pm.


# This solution interleaves a sufficiently long sequence of repetitions
# of Songs with a sufficiently long sequence of repetitions of Ads in its
# AdLoop initializer, such that no other methods of Loop need to be over-ridden
# and no new attributes are needed!

# Decision here was not to shuffle the Ads.

from a6_classes import Song, Mix, Loop


class Ad(Song):
    """An Ad should NOT be added to the Song class's all_of_em variable,
    because they aren't legitimate Songs. For instance, we wouldn't want them
    displayed in a catalog of Songs.

    Every Ad has artist "[Advertiser]" (literally, with the square brackets.)

    Class attribute:
        all_of_em: list of all Ads that are created.

    Instance attributes:
        includes those for Songs.

    """
    all_of_em = []


    def __init__(self, t, yt = None):
        super().__init__(t, "[Advertiser]", yt)
        Song.all_of_em.remove(self)
        Ad.all_of_em.append(self)


class AdLoop(Loop):
    """
    Like a Loop, except that it plays an Ad after every Song.  The Ads should
    appear in the order they occur in Ad.all_of_em.

    Example: suppose the source Mix has only two Songs, s1 and s2, and
    Ad.all_of_em has Ads a1, a2, a3, a4, a5.
    Then, the sequence of Songs occurring if one ran the play() method multiple
    times is:
       s1 a1 s2 a2 s1 a3 s2 a4 s1 a5 s2 a1 s1 a2 ... # typo fixed 4/30 (s2->s1)

    Instance attributes:
        Includes those of Loops.

    """
    def __init__(self, m, shuffle=False):
        super().__init__(m, shuffle)
        # We extend alist and slist by repetition so that both end up with the same length
        alist = Ad.all_of_em * len(self.slist)
        slist = self.slist * len(Ad.all_of_em)
        self.slist = []
        # Merge the two lists, alternating between Songs and Ads
        for i in range(len(alist)):
            self.slist += [slist[i], alist[i]]
